/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.se.eip.patterns;

import java.net.URI;
import java.util.logging.Logger;

import javax.jbi.messaging.MessagingException;

import org.ow2.petals.component.framework.util.Exchange;
import org.ow2.petals.se.eip.ExchangeContext;

/**
 *
 * Created on 17 janv. 08
 *
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since
 *
 */
public abstract class AbstractPattern implements Pattern {

	protected Logger logger;



	/*
	 * (non-Javadoc)
	 *
	 * @see org.ow2.petals.se.eip.patterns.Pattern#processPattern(org.ow2.petals.component.framework.util.Exchange,
	 *      org.ow2.petals.se.eip.ExchangeContext)
	 */
	public final void processPattern(Exchange exchange, ExchangeContext context) {
		this.logger = context.getLogger();
		this.logger.info(getPatternName());

		URI pattern = null;
		try {
			pattern = exchange.getExchangePattern();
		} catch (MessagingException e) {
			exchange.setError(new Exception(e));
		}

		if (!validateMEP(pattern)) {
			String fault = "The " + getPatternName()
					+ " pattern does not support " + pattern + " pattern";
			exchange.setError(new Exception(fault));

		} else {
			try {
				this.process(exchange, context);
			} catch (MessagingException e) {
				logger.severe(e.getMessage());
				exchange.setError(e);
			}
		}
	}

	/**
	 * Process the message.
	 *
	 * @param exchange
	 * @param context
	 */
	public abstract void process(Exchange exchange, ExchangeContext context)
			throws MessagingException;

	/**
	 * Validate the MEP for the pattern. This method is called on each message
	 * exchange in order to validate that the MEP is compatible.
	 *
	 * @param mep
	 *            the mep of the incoming message.
	 * @return true if the mep is compatible, false otherwise.
	 */
	protected abstract boolean validateMEP(URI mep);

	/**
	 * Get the pattern name.
	 *
	 * @return
	 */
	protected abstract String getPatternName();




}
