/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.se.eip.patterns;

import java.net.URI;
import java.util.List;
import java.util.logging.Level;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;

import org.ow2.petals.component.framework.Constants.MEPConstants;
import org.ow2.petals.component.framework.util.Exchange;
import org.ow2.petals.jbi.descriptor.Consumes;
import org.ow2.petals.se.eip.ExchangeContext;

/**
 * 
 * Dispatch the exchange to all Consumes services defined in the service unit.
 * <br>
 * <br> - Accept InOnly and RobustInOnly <br> - If a Fault or Error occurs on a
 * call, stop the process and return the Fault or Error Note : The exchange will
 * be sent back automatically by the framework
 * 
 * @author alouis, chamerling - eBM WebSourcing
 * 
 */
public class Dispatcher extends AbstractPattern {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ow2.petals.se.eip.patterns.Pattern#processPattern(org.ow2.petals.component.framework.util.Exchange,
	 *      org.ow2.petals.se.eip.ExchangeContext)
	 */
	public void process(Exchange exchange, ExchangeContext context) {
		Exchange originalExchange = exchange;
		List<Consumes> consumesList = context.getSUConsumes(originalExchange
				.getEndpoint());

		sendAsyncExchanges(consumesList, exchange, context);
	}

	/**
	 * Check InOnly or RobustInOnly pattern. <br>
	 * 
	 * @param uri
	 * @return true if MEP is InOnly or RobustInOnly, false otherwise
	 */
	protected boolean validateMEP(URI uri) {
		return MEPConstants.IN_ONLY_PATTERN.equals(uri)
				|| MEPConstants.ROBUST_IN_ONLY_PATTERN.equals(uri);
	}

	/**
	 * Send a copy of the exchange's IN message to all the services to consume.
	 * If an error occurs, set a Fault on the original exchange (for robust
	 * exchange). MEP used is InOnly
	 * 
	 * @param consumesList
	 * @param exchange
	 * @param context
	 */
	protected void sendAsyncExchanges(List<Consumes> consumesList,
			Exchange exchange, ExchangeContext context) {

		try {
			NormalizedMessage inMessage = exchange.getInMessage();
			// send all exchanges asynchronously, inonly
			for (Consumes consumedService : consumesList) {
				Exchange tmpExchange = PatternHelper.createExchange(
						consumedService, MEPConstants.IN_ONLY_PATTERN.value(),
						inMessage, context);
				logger.info(
						"send exchange to ...\n - itf="
								+ consumedService.getInterfaceName()
								+ "\n - srv="
								+ consumedService.getServiceName()
								+ "\n - ep ="
								+ consumedService.getEndpointName());
				context.sendAsync(tmpExchange);
			}
		} catch (JBIException e) {
			logger.log(Level.SEVERE, e.getMessage(), e);
			try {
				exchange.setFault(e);
			} catch (MessagingException ex) {
				logger.severe(ex.getMessage());
				exchange.setError(ex);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ow2.petals.se.eip.patterns.Pattern#init()
	 */
	public void init() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.ow2.petals.se.eip.patterns.AbstractPattern#getPatternName()
	 */
	@Override
	protected String getPatternName() {
		return "Dispatcher";
	}
}
