/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.se.eip.patterns;

import java.net.URI;
import java.util.List;
import java.util.logging.Level;

import javax.jbi.JBIException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;
import javax.xml.xpath.XPathExpressionException;

import org.ow2.petals.component.framework.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.util.Exchange;
import org.ow2.petals.component.framework.util.Extensions;
import org.ow2.petals.jbi.descriptor.Consumes;
import org.ow2.petals.se.eip.ExchangeContext;
import org.w3c.dom.Document;

/**
 * Route the exchange on the correct Consumes services defined in the service
 * unit. <br>
 * Return the result of the service, if any.<br>
 *
 * @author alouis, chamerling - eBM WebSourcing
 *
 */
public class Router extends AbstractPattern {

	public static final String CONDITION = "condition";

	/*
	 * (non-Javadoc)
	 * @see org.ow2.petals.se.eip.patterns.AbstractPattern#process(org.ow2.petals.component.framework.util.Exchange, org.ow2.petals.se.eip.ExchangeContext)
	 */
	public void process(Exchange exchange, ExchangeContext context) {
		try {
			List<Consumes> consumesList = context.getSUConsumes(exchange
					.getEndpoint());

			QName inputOperation = exchange.getOperation();

			// there must be only 2 'consumes' defined in the SU
			int count = consumesList.size();
			if (count <= 0 || count > 2) {
				throw new PEtALSCDKException(
						"only 2 <Consumes> elements must be defined in the service-unit.");
			} else {
				Exchange originalExchange = exchange;
				// retrieve the condition
				String condition = context.getExtensions().getValue(CONDITION);
				// evaluate the IN message
				Document inMessage = originalExchange
						.getInMessageContentAsDocument();
				logger.info("Evaluate condition: \"" + condition);
				boolean eval = PatternHelper.evaluateXPathCondition(condition, inMessage
						.getFirstChild(), context);
				// forward the exchange to the correct service, depending on the
				// test result
				Consumes consumedService = consumesList.get(eval ? 0 : 1);
				Exchange routedExchange = PatternHelper.createExchange(
						consumedService, originalExchange.getPattern(),
						originalExchange.getInMessage(), context);
				logger.info(
						"send exchange to ...\n - itf="
								+ consumedService.getInterfaceName()
								+ "\n - srv="
								+ consumedService.getServiceName()
								+ "\n - ep ="
								+ consumedService.getEndpointName());

				Extensions extensions = new Extensions(consumedService
						.getExtensions());
				PatternHelper.setInputOperation(routedExchange, extensions,
						inputOperation);

				context.sendSync(routedExchange);

				// process the response
				NormalizedMessage receivedMessage = PatternHelper
						.processSentExchange(routedExchange, originalExchange,
								context);
				// if a response has been returned, set it on the original
				// exchange
				// otherwise, the 'processSentExchange()' has manage the
				// exchange
				if (receivedMessage != null) {
					originalExchange.setOutMessage(receivedMessage);
				}
			}
		} catch (JBIException e) {
			logger.log(Level.SEVERE, e.getMessage(), e);
			exchange.setError(e);
		} catch (XPathExpressionException e) {
			logger.log(Level.SEVERE, e.getMessage(), e);
			exchange.setError(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.ow2.petals.se.eip.patterns.AbstractPattern#getPatternName()
	 */
	@Override
	protected String getPatternName() {
		return "Router";
	}

	/*
	 * (non-Javadoc)
	 * @see org.ow2.petals.se.eip.patterns.AbstractPattern#validateMEP(java.net.URI)
	 */
	@Override
	protected boolean validateMEP(URI mep) {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * @see org.ow2.petals.se.eip.patterns.Pattern#init()
	 */
	public void init() {
	}
}
